@@ -1,4 +1,4 @@
-require File.join(File.dirname(File.expand_path(__FILE__)), "spec_helper")
+require_relative "spec_helper"
 
 describe "Sequel Mock Adapter" do
   it "should have an adapter method" do
@@ -7,7 +7,39 @@ describe "Sequel Mock Adapter" do
     db.adapter_scheme.must_equal :mock
   end
 
+  it "should support registering mock adapter type" do
+    begin
+      Module.new do
+        Sequel::Database.set_shared_adapter_scheme(:foo, self)
+        
+        def self.mock_adapter_setup(db)
+          db.instance_variable_set(:@foo, :foo)
+        end
+
+        module self::DatabaseMethods
+          def foo
+            @foo
+          end
+        end
+
+        module self::DatasetMethods
+          def foo
+            db.foo
+          end
+        end
+      end
+
+      Sequel.connect('mock://foo') do |db|
+        db.foo.must_equal :foo
+        db.dataset.foo.must_equal :foo
+      end
+    ensure
+      Sequel.synchronize{Sequel::SHARED_ADAPTER_MAP.delete(:foo)}
+    end
+  end
+
   it "should have constructor accept no arguments" do
+    Sequel.require 'adapters/mock'
     Sequel::Mock::Database.new.must_be_kind_of(Sequel::Mock::Database)
   end
 
@@ -25,8 +57,8 @@ describe "Sequel Mock Adapter" do
   end
 
   it "should return nil for insert/execute_insert by default" do
-    Sequel.mock[:t].insert(:a=>1).must_equal nil
-    Sequel.mock.execute_insert('INSERT INTO a () DEFAULT VALUES').must_equal nil
+    Sequel.mock[:t].insert(:a=>1).must_be_nil
+    Sequel.mock.execute_insert('INSERT INTO a () DEFAULT VALUES').must_be_nil
   end
 
   it "should be able to set the rows returned by each using :fetch option with a single hash" do
@@ -118,13 +150,13 @@ describe "Sequel Mock Adapter" do
     proc{db[:t].all}.must_raise(Sequel::DatabaseError)
   end
 
-  it "should be able to set the rows returned by each on a per dataset basis using _fetch" do
+  it "should be able to set the rows returned by each on a per dataset basis using with_fetch" do
     rs = []
     db = Sequel.mock(:fetch=>{:a=>1})
     ds = db[:t]
     ds.each{|r| rs << r}
     rs.must_equal [{:a=>1}]
-    ds._fetch = {:b=>2}
+    ds = ds.with_fetch(:b=>2)
     ds.each{|r| rs << r}
     rs.must_equal [{:a=>1}, {:b=>2}]
   end
@@ -191,7 +223,7 @@ describe "Sequel Mock Adapter" do
     ds = db[:t]
     ds.update(:a=>1).must_equal 2
     ds.delete.must_equal 2
-    ds.numrows = 3
+    ds = ds.with_numrows(3)
     ds.update(:a=>1).must_equal 3
     ds.delete.must_equal 3
   end
@@ -215,7 +247,7 @@ describe "Sequel Mock Adapter" do
     db[:t].insert(:a=>1).must_equal 1
     db[:t].insert(:a=>1).must_equal 3
     db[:t].insert(:a=>1).must_equal 5
-    db[:t].insert(:a=>1).must_equal nil
+    db[:t].insert(:a=>1).must_be_nil
   end
 
   it "should be able to set the autogenerated primary key returned by insert using :autoid option as a proc" do
@@ -241,7 +273,7 @@ describe "Sequel Mock Adapter" do
     db = Sequel.mock(:autoid=>[1, proc{|s| 2}, nil, ArgumentError])
     db[:t].insert.must_equal 1
     db[:t].insert.must_equal 2
-    db[:t].insert.must_equal nil
+    db[:t].insert.must_be_nil
     proc{db[:t].insert}.must_raise(Sequel::DatabaseError)
   end
 
@@ -257,7 +289,7 @@ describe "Sequel Mock Adapter" do
     db = Sequel.mock(:autoid=>1)
     ds = db[:t]
     ds.insert(:a=>1).must_equal 1
-    ds.autoid = 5
+    ds = ds.with_autoid(5)
     ds.insert(:a=>1).must_equal 5
     ds.insert(:a=>1).must_equal 6
     db[:t].insert(:a=>1).must_equal 2
@@ -278,7 +310,7 @@ describe "Sequel Mock Adapter" do
   it "should be able to set the columns to set in the dataset as an array of arrays of symbols" do
     db = Sequel.mock(:columns=>[[:a, :b], [:c, :d]])
     db[:t].columns.must_equal [:a, :b]
-    db[:t].columns.must_equal [:c, :d]
+    db[:x].columns.must_equal [:c, :d]
   end
 
   it "should be able to set the columns to set in the dataset as a proc" do
@@ -291,7 +323,7 @@ describe "Sequel Mock Adapter" do
     db = Sequel.mock
     db.columns = [[:a, :b], [:c, :d]]
     db[:t].columns.must_equal [:a, :b]
-    db[:t].columns.must_equal [:c, :d]
+    db[:x].columns.must_equal [:c, :d]
   end
 
   it "should raise Error if given an invalid columns" do
@@ -304,14 +336,13 @@ describe "Sequel Mock Adapter" do
 
   it "should allow overriding of server_version" do
     db = Sequel.mock
-    db.server_version.must_equal nil
+    db.server_version.must_be_nil
     db.server_version = 80102
     db.server_version.must_equal 80102
   end
 
-  it "should not have identifier input/output methods by default" do
-    Sequel.mock.send(:identifier_input_method_default).must_equal nil
-    Sequel.mock.send(:identifier_output_method_default).must_equal nil
+  it "should not fold to uppercase by default" do
+    Sequel.mock.send(:folds_unquoted_identifiers_to_uppercase?).must_equal false
   end
 
   it "should keep a record of all executed SQL in #sqls" do
@@ -418,32 +449,21 @@ describe "Sequel Mock Adapter" do
   end
 
   it "should be able to load dialects based on the database name" do
-    begin
-      qi = class Sequel::Database; @quote_identifiers; end
-      ii = class Sequel::Database; @identifier_input_method; end
-      io = class Sequel::Database; @identifier_output_method; end
-      Sequel.quote_identifiers = nil
-      class Sequel::Database; @identifier_input_method=nil; end
-      class Sequel::Database; @identifier_output_method=nil; end
-      Sequel.mock(:host=>'access').select(Date.new(2011, 12, 13)).sql.must_equal 'SELECT #2011-12-13#'
-      Sequel.mock(:host=>'cubrid').from(:a).offset(1).sql.must_equal 'SELECT * FROM "a" LIMIT 1,4294967295'
-      Sequel.mock(:host=>'db2').select(1).sql.must_equal 'SELECT 1 FROM "SYSIBM"."SYSDUMMY1"'
-      Sequel.mock(:host=>'firebird')[:a].distinct.limit(1, 2).sql.must_equal 'SELECT DISTINCT FIRST 1 SKIP 2 * FROM "A"'
-      Sequel.mock(:host=>'informix')[:a].distinct.limit(1, 2).sql.must_equal 'SELECT SKIP 2 FIRST 1 DISTINCT * FROM A'
-      Sequel.mock(:host=>'mssql')[:a].full_text_search(:b, 'c').sql.must_equal "SELECT * FROM [A] WHERE (CONTAINS ([B], 'c'))"
-      Sequel.mock(:host=>'mysql')[:a].full_text_search(:b, 'c').sql.must_equal "SELECT * FROM `a` WHERE (MATCH (`b`) AGAINST ('c'))"
-      Sequel.mock(:host=>'oracle')[:a].limit(1).sql.must_equal 'SELECT * FROM (SELECT * FROM "A") "T1" WHERE (ROWNUM <= 1)'
-      Sequel.mock(:host=>'postgres')[:a].full_text_search(:b, 'c').sql.must_equal "SELECT * FROM \"a\" WHERE (to_tsvector(CAST('simple' AS regconfig), (COALESCE(\"b\", ''))) @@ to_tsquery(CAST('simple' AS regconfig), 'c'))"
-      Sequel.mock(:host=>'sqlanywhere').from(:a).offset(1).sql.must_equal 'SELECT TOP 2147483647 START AT (1 + 1) * FROM "A"'
-      Sequel.mock(:host=>'sqlite')[:a___b].sql.must_equal "SELECT * FROM `a` AS 'b'"
-    ensure
-      Sequel.quote_identifiers = qi
-      Sequel::Database.send(:instance_variable_set, :@identifier_input_method, ii)
-      Sequel::Database.send(:instance_variable_set, :@identifier_output_method, io)
-    end
+    Sequel.mock(:host=>'access').select(Date.new(2011, 12, 13)).sql.must_equal 'SELECT #2011-12-13#'
+    Sequel.mock(:host=>'db2').select(1).sql.must_equal 'SELECT 1 FROM "SYSIBM"."SYSDUMMY1"'
+    Sequel.mock(:host=>'mssql')[:A].full_text_search(:B, 'C').with_quote_identifiers(false).sql.must_equal "SELECT * FROM A WHERE (CONTAINS (B, 'C'))"
+    Sequel.mock(:host=>'mysql')[:a].full_text_search(:b, 'c').with_quote_identifiers(false).sql.must_equal "SELECT * FROM a WHERE (MATCH (b) AGAINST ('c'))"
+    Sequel.mock(:host=>'oracle')[:a].limit(1).with_quote_identifiers(false).sql.upcase.must_equal 'SELECT * FROM (SELECT * FROM A) T1 WHERE (ROWNUM <= 1)'
+    Sequel.mock(:host=>'postgres')[:a].full_text_search(:b, 'c').with_quote_identifiers(false).sql.must_equal "SELECT * FROM a WHERE (to_tsvector(CAST('simple' AS regconfig), (COALESCE(b, ''))) @@ to_tsquery(CAST('simple' AS regconfig), 'c'))"
+    Sequel.mock(:host=>'sqlanywhere').from(:A).offset(1).with_quote_identifiers(false).sql.must_equal 'SELECT TOP 2147483647 START AT (1 + 1) * FROM A'
+    Sequel.mock(:host=>'sqlite')[Sequel[:a].as(:b)].with_quote_identifiers(false).sql.must_equal "SELECT * FROM a AS 'b'"
   end
 
-  it "should automatically set version for adapters nedding versions" do
+  it "should be able to mock schema calls" do
+    Sequel.mock(:host=>'mysql', :fetch=>{:Field=>'a'}).schema(:a).first.first.must_equal :a
+  end
+
+  it "should automatically set version for adapters needing versions" do
     Sequel.mock(:host=>'postgres').server_version.must_be :>=, 90400
     Sequel.mock(:host=>'mssql').server_version.must_be :>=, 11000000
     Sequel.mock(:host=>'mysql').server_version.must_be :>=, 50617
@@ -463,3 +483,240 @@ describe "Sequel Mock Adapter" do
     Sequel.mock(:host=>'oracle').create_table(:a){String :b}
   end
 end
+
+describe "PostgreSQL support" do
+  before do
+    @db = Sequel.mock(:host=>'postgres')
+  end
+
+  it "should create an unlogged table" do
+    @db.create_table(:unlogged_dolls, :unlogged => true){text :name}
+    @db.sqls.must_equal ['CREATE UNLOGGED TABLE "unlogged_dolls" ("name" text)']
+  end
+
+  it "should support spatial indexes" do
+    @db.alter_table(:posts){add_spatial_index [:geom]}
+    @db.sqls.must_equal ['CREATE INDEX "posts_geom_index" ON "posts" USING gist ("geom")']
+  end
+
+  it "should support indexes with index type" do
+    @db.alter_table(:posts){add_index :p, :type => 'gist'}
+    @db.sqls.must_equal ['CREATE INDEX "posts_p_index" ON "posts" USING gist ("p")']
+  end
+
+  it "should have #transaction support various types of synchronous options" do
+    @db.transaction(:synchronous=>:on){}
+    @db.transaction(:synchronous=>true){}
+    @db.transaction(:synchronous=>:off){}
+    @db.transaction(:synchronous=>false){}
+    @db.sqls.grep(/synchronous/).must_equal ["SET LOCAL synchronous_commit = on", "SET LOCAL synchronous_commit = on", "SET LOCAL synchronous_commit = off", "SET LOCAL synchronous_commit = off"]
+
+    @db.transaction(:synchronous=>nil){}
+    @db.sqls.must_equal ['BEGIN', 'COMMIT']
+
+    if @db.server_version >= 90100
+      @db.transaction(:synchronous=>:local){}
+      @db.sqls.grep(/synchronous/).must_equal ["SET LOCAL synchronous_commit = local"]
+
+      if @db.server_version >= 90200
+        @db.transaction(:synchronous=>:remote_write){}
+        @db.sqls.grep(/synchronous/).must_equal ["SET LOCAL synchronous_commit = remote_write"]
+      end
+    end
+  end
+
+  it "should have #transaction support read only transactions" do
+    @db.transaction(:read_only=>true){}
+    @db.transaction(:read_only=>false){}
+    @db.transaction(:isolation=>:serializable, :read_only=>true){}
+    @db.transaction(:isolation=>:serializable, :read_only=>false){}
+    @db.sqls.grep(/READ/).must_equal ["SET TRANSACTION READ ONLY", "SET TRANSACTION READ WRITE", "SET TRANSACTION ISOLATION LEVEL SERIALIZABLE READ ONLY", "SET TRANSACTION ISOLATION LEVEL SERIALIZABLE READ WRITE"]
+  end
+
+  it "should have #transaction support deferrable transactions" do
+    @db.transaction(:deferrable=>true){}
+    @db.transaction(:deferrable=>false){}
+    @db.transaction(:deferrable=>true, :read_only=>true){}
+    @db.transaction(:deferrable=>false, :read_only=>false){}
+    @db.transaction(:isolation=>:serializable, :deferrable=>true, :read_only=>true){}
+    @db.transaction(:isolation=>:serializable, :deferrable=>false, :read_only=>false){}
+    @db.sqls.grep(/DEF/).must_equal ["SET TRANSACTION DEFERRABLE", "SET TRANSACTION NOT DEFERRABLE", "SET TRANSACTION READ ONLY DEFERRABLE", "SET TRANSACTION READ WRITE NOT DEFERRABLE",  "SET TRANSACTION ISOLATION LEVEL SERIALIZABLE READ ONLY DEFERRABLE", "SET TRANSACTION ISOLATION LEVEL SERIALIZABLE READ WRITE NOT DEFERRABLE"]
+  end
+
+  it "should support creating indexes concurrently" do
+    @db.add_index :test, [:name, :value], :concurrently=>true
+    @db.sqls.must_equal ['CREATE INDEX CONCURRENTLY "test_name_value_index" ON "test" ("name", "value")']
+  end
+
+  it "should support dropping indexes concurrently" do
+    @db.drop_index :test, [:name, :value], :concurrently=>true, :name=>'tnv2'
+    @db.sqls.must_equal ['DROP INDEX CONCURRENTLY "tnv2"']
+  end
+
+  it "should use INSERT RETURNING for inserts" do
+    @db[:test5].insert(:value=>10)
+    @db.sqls.must_equal ['INSERT INTO "test5" ("value") VALUES (10) RETURNING "id"']
+  end
+
+  it "should support opclass specification" do
+    @db.alter_table(:posts){add_index(:user_id, :opclass => :int4_ops, :type => :btree)}
+    @db.sqls.must_equal ['CREATE INDEX "posts_user_id_index" ON "posts" USING btree ("user_id" int4_ops)']
+  end
+
+  it 'should quote NaN' do
+    nan = 0.0/0.0
+    @db[:test5].insert_sql(:value => nan).must_equal %q{INSERT INTO "test5" ("value") VALUES ('NaN')}
+  end
+
+  it 'should quote +Infinity' do
+    inf = 1.0/0.0
+    @db[:test5].insert_sql(:value => inf).must_equal %q{INSERT INTO "test5" ("value") VALUES ('Infinity')}
+  end
+
+  it 'should quote -Infinity' do
+    inf = -1.0/0.0
+    @db[:test5].insert_sql(:value => inf).must_equal %q{INSERT INTO "test5" ("value") VALUES ('-Infinity')}
+  end
+
+  it "Dataset#insert_conflict should respect expressions in the target argument" do
+    @ds = @db[:ic_test]
+    @ds.insert_conflict(:target=>:a).insert_sql(1, 2, 3).must_equal "INSERT INTO \"ic_test\" VALUES (1, 2, 3) ON CONFLICT (\"a\") DO NOTHING"
+    @ds.insert_conflict(:target=>:c, :conflict_where=>{:c_is_unique=>true}).insert_sql(1, 2, 3).must_equal "INSERT INTO \"ic_test\" VALUES (1, 2, 3) ON CONFLICT (\"c\") WHERE (\"c_is_unique\" IS TRUE) DO NOTHING"
+    @ds.insert_conflict(:target=>[:b, :c]).insert_sql(1, 2, 3).must_equal "INSERT INTO \"ic_test\" VALUES (1, 2, 3) ON CONFLICT (\"b\", \"c\") DO NOTHING"
+    @ds.insert_conflict(:target=>[:b, Sequel.function(:round, :c)]).insert_sql(1, 2, 3).must_equal "INSERT INTO \"ic_test\" VALUES (1, 2, 3) ON CONFLICT (\"b\", round(\"c\")) DO NOTHING"
+    @ds.insert_conflict(:target=>[:b, Sequel.virtual_row{|o| o.round(:c)}]).insert_sql(1, 2, 3).must_equal "INSERT INTO \"ic_test\" VALUES (1, 2, 3) ON CONFLICT (\"b\", round(\"c\")) DO NOTHING"
+  end
+
+  it "should support creating and dropping foreign tables" do
+    @db.create_table(:t, :foreign=>:f, :options=>{:o=>1}){Integer :a}
+    @db.drop_table(:t, :foreign=>true)
+    @db.sqls.must_equal ['CREATE FOREIGN TABLE "t" ("a" integer) SERVER "f" OPTIONS (o \'1\')',
+      'DROP FOREIGN TABLE "t"']
+  end
+
+  it "#create_function and #drop_function should create and drop functions" do
+    @db.create_function('tf', 'SELECT 1', :returns=>:integer)
+    @db.drop_function('tf')
+    @db.sqls.map{|s| s.gsub(/\s+/, ' ').strip}.must_equal ["CREATE FUNCTION tf() RETURNS integer LANGUAGE SQL AS 'SELECT 1'",
+      'DROP FUNCTION tf()']
+  end
+
+  it "#create_function and #drop_function should support options" do
+    @db.create_function('tf', 'SELECT $1 + $2', :args=>[[:integer, :a], :integer], :replace=>true, :returns=>:integer, :language=>'SQL', :behavior=>:immutable, :strict=>true, :security_definer=>true, :cost=>2, :set=>{:search_path => 'public'})
+    @db.drop_function('tf', :if_exists=>true, :cascade=>true, :args=>[[:integer, :a], :integer])
+    @db.sqls.map{|s| s.gsub(/\s+/, ' ').strip}.must_equal ["CREATE OR REPLACE FUNCTION tf(a integer, integer) RETURNS integer LANGUAGE SQL IMMUTABLE STRICT SECURITY DEFINER COST 2 SET search_path = public AS 'SELECT $1 + $2'",
+      'DROP FUNCTION IF EXISTS tf(a integer, integer) CASCADE']
+  end
+
+  it "#create_language and #drop_language should create and drop languages" do
+    @db.create_language(:plpgsql)
+    @db.create_language(:plpgsql, :replace=>true, :trusted=>true, :handler=>:a, :validator=>:b)
+    @db.drop_language(:plpgsql)
+    @db.drop_language(:plpgsql, :if_exists=>true, :cascade=>true)
+    @db.sqls.map{|s| s.gsub(/\s+/, ' ').strip}.must_equal ['CREATE LANGUAGE plpgsql',
+      'CREATE OR REPLACE TRUSTED LANGUAGE plpgsql HANDLER a VALIDATOR b',
+      'DROP LANGUAGE plpgsql',
+      'DROP LANGUAGE IF EXISTS plpgsql CASCADE']
+  end
+
+  it "#create_schema and #drop_schema should create and drop schemas" do
+    @db.create_schema(:sequel)
+    @db.create_schema(:sequel, :if_not_exists=>true, :owner=>:foo)
+    @db.drop_schema(:sequel)
+    @db.drop_schema(:sequel, :if_exists=>true, :cascade=>true)
+    @db.sqls.must_equal ['CREATE SCHEMA "sequel"',
+      'CREATE SCHEMA IF NOT EXISTS "sequel" AUTHORIZATION "foo"',
+      'DROP SCHEMA "sequel"',
+      'DROP SCHEMA IF EXISTS "sequel" CASCADE']
+  end
+
+  it "#create_trigger and #drop_trigger should create and drop triggers" do
+    @db.create_trigger(:test, :identity, :tf, :each_row=>true)
+    @db.create_trigger(:test, :identity, :tf, :after=>true, :events=>:insert, :args=>[1, 'a'])
+    @db.create_trigger(:test, :identity, :tf, :each_row=>true, :when=> {Sequel[:new][:name] => 'b'})
+    @db.drop_trigger(:test, :identity)
+    @db.drop_trigger(:test, :identity, :if_exists=>true, :cascade=>true)
+    @db.sqls.map{|s| s.gsub(/\s+/, ' ').strip}.must_equal ['CREATE TRIGGER identity BEFORE INSERT OR UPDATE OR DELETE ON "test" FOR EACH ROW EXECUTE PROCEDURE tf()',
+      'CREATE TRIGGER identity AFTER INSERT ON "test" EXECUTE PROCEDURE tf(1, \'a\')',
+      %q{CREATE TRIGGER identity BEFORE INSERT OR UPDATE OR DELETE ON "test" FOR EACH ROW WHEN ("new"."name" = 'b') EXECUTE PROCEDURE tf()},
+      'DROP TRIGGER identity ON "test"',
+      'DROP TRIGGER IF EXISTS identity ON "test" CASCADE']
+  end
+end
+
+describe "MySQL support" do
+  before do
+    @db = Sequel.mock(:host=>'mysql')
+  end
+
+  it "should support spatial indexes" do
+    @db.alter_table(:posts){add_spatial_index [:geom]}
+    @db.sqls.must_equal ['CREATE SPATIAL INDEX `posts_geom_index` ON `posts` (`geom`)']
+  end
+
+  it "should support fulltext indexes and full_text_search" do
+    @db.alter_table(:posts){add_full_text_index :title; add_full_text_index [:title, :body]}
+    @db.sqls.must_equal [ "CREATE FULLTEXT INDEX `posts_title_index` ON `posts` (`title`)", "CREATE FULLTEXT INDEX `posts_title_body_index` ON `posts` (`title`, `body`)" ]
+  end
+
+  it "should support indexes with index type" do
+    @db.alter_table(:posts){add_index :id, :type => :btree}
+    @db.sqls.must_equal ["CREATE INDEX `posts_id_index` USING btree ON `posts` (`id`)"]
+  end
+end
+
+describe "SQLite support" do
+  before do
+    @db = Sequel.mock(:host=>'sqlite')
+  end
+
+  it "should use a string literal for Sequel[:col].as(:alias)" do
+    @db.literal(Sequel[:c].as(:a)).must_equal "`c` AS 'a'"
+  end
+
+  it "should use a string literal for Sequel[:table][:col].as(:alias)" do
+    @db.literal(Sequel[:t][:c].as(:a)).must_equal "`t`.`c` AS 'a'"
+  end
+
+  it "should use a string literal for :column.as(:alias)" do
+    @db.literal(Sequel.as(:c, :a)).must_equal "`c` AS 'a'"
+  end
+
+  it "should use a string literal in the SELECT clause" do
+    @db[:t].select(Sequel[:c].as(:a)).sql.must_equal "SELECT `c` AS 'a' FROM `t`"
+  end
+
+  it "should use a string literal in the FROM clause" do
+    @db[Sequel[:t].as(:a)].sql.must_equal "SELECT * FROM `t` AS 'a'"
+  end
+
+  it "should use a string literal in the JOIN clause" do
+    @db[:t].join_table(:natural, :j, nil, :table_alias=>:a).sql.must_equal "SELECT * FROM `t` NATURAL JOIN `j` AS 'a'"
+  end
+
+  it "should have support for various #transaction modes" do
+    @db.transaction{}
+    @db.transaction(:mode => :immediate){}
+    @db.transaction(:mode => :exclusive){}
+    @db.transaction(:mode => :deferred){}
+    @db.sqls.must_equal ["BEGIN", "COMMIT", "BEGIN IMMEDIATE TRANSACTION", "COMMIT", "BEGIN EXCLUSIVE TRANSACTION", "COMMIT", "BEGIN DEFERRED TRANSACTION", "COMMIT"]
+
+    @db.transaction_mode.must_be_nil
+    @db.transaction_mode = :immediate
+    @db.transaction_mode.must_equal :immediate
+    @db.transaction{}
+    @db.transaction(:mode => :exclusive){}
+    @db.sqls.must_equal ["BEGIN IMMEDIATE TRANSACTION", "COMMIT", "BEGIN EXCLUSIVE TRANSACTION", "COMMIT"]
+  end
+
+  it "should choose a temporary table name that isn't already used when dropping or renaming columns" do
+    exists = [true, true, false]
+    @db.define_singleton_method(:table_exists?){|x| exists.shift}
+    @db.drop_column(:test3, :i)
+    @db.sqls.grep(/ALTER/).must_equal ["ALTER TABLE `test3` RENAME TO `test3_backup2`"]
+
+    exists = [true, true, true, false]
+    @db.rename_column(:test3, :h, :i)
+    @db.sqls.grep(/ALTER/).must_equal ["ALTER TABLE `test3` RENAME TO `test3_backup3`"]
+  end
+end
